/// Copyright (c) Microsoft Corporation. 
/// 
/// The contents of this file are subject to the terms of the New BSD License. 
/// You may obtain a copy of the License at http://es5conformm.codeplex.com/license 
/// 
/// Alternatively the contents of this file may be used under the terms of the Microsoft Public License. 
/// You may obtain a copy of the License at http://www.microsoft.com/opensource/licenses.mspx#Ms-PL 

/*
Step 4 of defineProperty calls the [[DefineOwnProperty]] internal method
of O to define the property. For non-configurable properties, step 9a of
[[DefineOwnProperty]] rejects changing the kind of a property.
*/
var testName = "Object.defineProperty throws TypeError when changing non-configurable accessor properties to data properties";

function testcase() {
  var o = {};

  // create an accessor property; all other attributes default to false.
 
  // dummy getter
  var getter = function () { return 1; }
  var d1 = { get: getter, configurable: false };
  Object.defineProperty(o, "foo", d1);

  // changing "foo" to be a data property should fail, since [[Configurable]]
  // on the original property will be false.
  var desc = { value: 101 };

  try {
    Object.defineProperty(o, "foo", desc);
  }
  catch (e) {
    if (e instanceof TypeError) {
      // the property should remain an accessor property.
      var d2 = Object.getOwnPropertyDescriptor(o, "foo");
      if (d2.get === getter &&
          d2.configurable === false) {
        return true;
      }
    }
  }
}

function prereq() {
  return fnExists(Object.defineProperty) && fnExists(Object.getOwnPropertyDescriptor);
}

sth_addTest(testName, testcase, prereq);