/// Copyright (c) Microsoft Corporation. 
/// 
/// The contents of this file are subject to the terms of the New BSD License. 
/// You may obtain a copy of the License at http://es5conformm.codeplex.com/license 
/// 
/// Alternatively the contents of this file may be used under the terms of the Microsoft Public License. 
/// You may obtain a copy of the License at http://www.microsoft.com/opensource/licenses.mspx#Ms-PL 

/*
Step 4 of defineProperty calls the [[DefineOwnProperty]] internal method
of O to define the property. For configurable properties, step 9b of
[[DefineOwnProperty]] permits changing the kind of a property.
*/
var testName = "Object.defineProperty permits changing data property to accessor property for configurable properties";

function testcase() {
  var o = {};

  // create a data property. In this case,
  // [[Enumerable]] and [[Configurable]] are true
  o["foo"] = 101;

  // changing "foo" to be an accessor should succeed, since [[Configurable]]
  // on the original property will be true. Existing values of [[Configurable]]
  // and [[Enumerable]] need to be preserved and the rest need to be set to
  // their default values

  // dummy getter
  var getter = function () { return 1; }
  var d1 = { get: getter };
  Object.defineProperty(o, "foo", d1);

  var d2 = Object.getOwnPropertyDescriptor(o, "foo");

  if (d2.get === getter &&
      d2.enumerable === true &&
      d2.configurable === true) {
    return true;
  }
}

function prereq() {
  return fnExists(Object.defineProperty) && fnExists(Object.getOwnPropertyDescriptor);
}

sth_addTest(testName, testcase, prereq);