/// Copyright (c) Microsoft Corporation. 
/// 
/// The contents of this file are subject to the terms of the New BSD License. 
/// You may obtain a copy of the License at http://es5conformm.codeplex.com/license 
/// 
/// Alternatively the contents of this file may be used under the terms of the Microsoft Public License. 
/// You may obtain a copy of the License at http://www.microsoft.com/opensource/licenses.mspx#Ms-PL 

/*
Step 4 of defineProperty calls the [[DefineOwnProperty]] internal method
of O to define the property. For configurable properties, step 9c of
[[DefineOwnProperty]] permits changing the kind of a property.
*/
var testName = "Object.defineProperty permits changing accessor property to data property for configurable properties";

function testcase() {
  var o = {};

  // define an accessor property
  // dummy getter
  var getter = function () { return 1; }
  var d1 = { get: getter, configurable: true };
  Object.defineProperty(o, "foo", d1);

  // changing "foo" to be a data valued property should succeed, since
  // [[Configurable]] on the original property will be true. Existing
  // values of [[Configurable]] and [[Enumerable]] need to be preserved
  // and the rest need to be set to their default values.
  var desc = { value: 101 };
  Object.defineProperty(o, "foo", desc);
  var d2 = Object.getOwnPropertyDescriptor(o, "foo");

  if (d2.value === 101 &&
      d2.writable === false &&
      d2.enumerable === false &&
      d2.configurable === true) {
    return true;
  }
}

function prereq() {
  return fnExists(Object.defineProperty) && fnExists(Object.getOwnPropertyDescriptor);
}

sth_addTest(testName, testcase, prereq);