/// Copyright (c) Microsoft Corporation. 
/// 
/// The contents of this file are subject to the terms of the New BSD License. 
/// You may obtain a copy of the License at http://es5conformm.codeplex.com/license 
/// 
/// Alternatively the contents of this file may be used under the terms of the Microsoft Public License. 
/// You may obtain a copy of the License at http://www.microsoft.com/opensource/licenses.mspx#Ms-PL 

/*
Step 4 of defineProperty calls the [[DefineOwnProperty]] internal method
of O to define the property. Step 6 of [[DefineOwnProperty]] returns if
every field of desc also occurs in current and every field in desc has
the same value as current.
*/
var testName = "Object.defineProperty is no-op if current and desc are the same accessor desc";

function testcase() {
  function sameAccessorDescriptorValues(d1, d2) {
    return (d1.get == d2.get &&
            d1.enumerable == d2.enumerable &&
            d1.configurable == d2.configurable);
  }

  var o = {};

  // create an accessor property with the following attributes:
  // enumerable: true, configurable: true
  var desc = {
               get: function () {},
               enumerable: true,
               configurable: true
             };

  Object.defineProperty(o, "foo", desc);

  // query for, and save, the desc. A subsequent call to defineProperty
  // with the same desc should not disturb the property definition.
  var d1 = Object.getOwnPropertyDescriptor(o, "foo");  

  // now, redefine the property with the same descriptor
  // the property defintion should not get disturbed.
  Object.defineProperty(o, "foo", desc);

  var d2 = Object.getOwnPropertyDescriptor(o, "foo"); 

  if (sameAccessorDescriptorValues(d1, d2) === true) {
    return true;
  }
}

function prereq() {
  return fnExists(Object.defineProperty) && fnExists(Object.getOwnPropertyDescriptor);
}

sth_addTest(testName, testcase, prereq);